==> cx88-1.4.4.tar.gz: cx88-1.4.4/client/drivers/wintv_pci_fm.cpp <==
/*-
 * Copyright 2011 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <unistd.h>
#include <sys/errno.h>
#include <iostream>
#include "fmd1216me.h"
#include "tda9887.h"
#include "cx88_debug.h"
#include "wintv_pci_fm.h"

namespace cx88_analog_driver_factory
{

   const cx88_source wintv_pci_fm::m_sources[] =
      {{"Tuner",       "RF input",                            cx88_driver::CX88_SOURCE_DEFAULT},
       {"SVideo",      "S-video input/line-in audio",         wintv_pci_fm::PCIFM_SOURCE_SVIDEO},
       {"Composite",   "Composite video input/line-in audio", wintv_pci_fm::PCIFM_SOURCE_COMPOSITE | wintv_pci_fm::PCIFM_SOURCE_LINEIN},
       {"CompositeRF", "Composite video input/tuner audio",   wintv_pci_fm::PCIFM_SOURCE_COMPOSITE},
       {"Line-In",     "Line-in audio",                       wintv_pci_fm::PCIFM_SOURCE_LINEIN},
       {NULL,          NULL,                                  0}};

   int wintv_pci_fm::create_i2c_drivers(void)
   {
      m_i2c_devs[1].driver = new tda9887(m_tunerconf, *(m_i2c_devs[1].dev),
         TDA9887_OPTION_PORT1_ENABLE | TDA9887_OPTION_RADIO_PORT1_DISABLE | TDA9887_OPTION_PORT2_ENABLE);
      m_i2c_devs[0].driver = new fmd1216me(m_tunerconf, *(m_i2c_devs[0].dev));
      if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL))
      {
         CX88ERR << "Error initializing tuner driver for WinTV PCI-FM" << std::endl;
         return ENOMEM;
      }
      return 0;
   }

   void wintv_pci_fm::destroy_i2c_drivers(void)
   {
      delete m_i2c_devs[0].driver;
      delete m_i2c_devs[1].driver;
   }

   int wintv_pci_fm::create_i2c_devices(void)
   {
      m_i2c_devs[0].conf.in_addr = 0x61;
      m_i2c_devs[0].source_mask = CX88_SOURCE_DEFAULT;
      m_i2c_devs[1].conf.in_addr = 0x43;
      m_i2c_devs[1].source_mask = CX88_SOURCE_DEFAULT;
      return 0;
   }

   int wintv_pci_fm::select_source(uint32_t source_mask)
   {
      switch (source_mask)
      {
         case CX88_SOURCE_DEFAULT:
            m_input_mux = CX88_VIDEO_INPUT_MUX1;
            m_input_type = CX88_VIDEO_INPUT_COMPOSITE;
            m_gpio = 0x0;
            break;
         case PCIFM_SOURCE_SVIDEO:
            m_input_mux = CX88_VIDEO_INPUT_MUX3;
            m_input_type = CX88_VIDEO_INPUT_SVIDEO;
            m_gpio = 0x2;
            break;
         case (PCIFM_SOURCE_COMPOSITE | PCIFM_SOURCE_LINEIN):
            m_input_mux = CX88_VIDEO_INPUT_MUX2;
            m_input_type = CX88_VIDEO_INPUT_COMPOSITE;
            m_gpio = 0x2;
            break;
         case PCIFM_SOURCE_COMPOSITE:
            m_input_mux = CX88_VIDEO_INPUT_MUX2;
            m_input_type = CX88_VIDEO_INPUT_COMPOSITE;
            m_gpio = 0x0;
            break;
         case PCIFM_SOURCE_LINEIN:
            m_gpio = 0x2;
            break;
         default:
            return EINVAL;
      }
      return 0;
   }

   int wintv_pci_fm::set_capture_options(cx88_video_capture &capture)
   {
      capture.in_input_source = m_input_mux;
      capture.in_input_type = m_input_type;
      return 0;
   }

   int wintv_pci_fm::set_channel_options(avb_channel &channel)
   {
      switch (channel.audio_format)
      {
         case AVB_AUDIO_FMT_FM_MONO:
         case AVB_AUDIO_FMT_FM_MONO_NON_USA:
         case AVB_AUDIO_FMT_FM_MONO_USA:
         case AVB_AUDIO_FMT_FM_STEREO:
         case AVB_AUDIO_FMT_FM_STEREO_NON_USA:
         case AVB_AUDIO_FMT_FM_STEREO_USA:
            m_gpio = 0x1;
         default:
            break;
      }
      int error = 0;
      cx88_common_gpio gpio;
      gpio.in_write_mask = 0xffffff;
      gpio.in_read_mask = 0x0;
      gpio.inout_gpio = m_gpio;
      if ((error = ioctl(m_fd, CX88_COMMON_GPIO, &gpio)) != 0)
      {
         CX88ERR << "Error configuring GPIO lines: " << strerror(errno) << std::endl;
      }
      return error;
   }
}

==> cx88-1.4.6.tar.gz: cx88-1.4.6/client/drivers/wintv_pci_fm.cpp <==
/*-
 * Copyright 2011 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <unistd.h>
#include <sys/errno.h>
#include <new>
#include <iostream>
#include "fmd1216me.h"
#include "tda9887.h"
#include "cx88_debug.h"
#include "wintv_pci_fm.h"

namespace cx88_analog_driver_factory
{

   const cx88_source wintv_pci_fm::m_sources[] =
      {{"Tuner",       "RF input",                            cx88_driver::CX88_SOURCE_DEFAULT},
       {"SVideo",      "S-video input/line-in audio",         wintv_pci_fm::PCIFM_SOURCE_SVIDEO},
       {"Composite",   "Composite video input/line-in audio", wintv_pci_fm::PCIFM_SOURCE_COMPOSITE | wintv_pci_fm::PCIFM_SOURCE_LINEIN},
       {"CompositeRF", "Composite video input/tuner audio",   wintv_pci_fm::PCIFM_SOURCE_COMPOSITE},
       {"Line-In",     "Line-in audio",                       wintv_pci_fm::PCIFM_SOURCE_LINEIN},
       {NULL,          NULL,                                  0}};

   int wintv_pci_fm::create_i2c_drivers(void)
   {
      m_i2c_devs[1].driver = new(std::nothrow) tda9887(m_tunerconf, *(m_i2c_devs[1].dev),
         TDA9887_OPTION_PORT1_ENABLE | TDA9887_OPTION_RADIO_PORT1_DISABLE | TDA9887_OPTION_PORT2_ENABLE);
      m_i2c_devs[0].driver = new(std::nothrow) fmd1216me(m_tunerconf, *(m_i2c_devs[0].dev));
      if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL))
      {
         CX88ERR << "Error initializing tuner driver for WinTV PCI-FM" << std::endl;
         return ENOMEM;
      }
      return 0;
   }

   void wintv_pci_fm::destroy_i2c_drivers(void)
   {
      delete m_i2c_devs[0].driver;
      delete m_i2c_devs[1].driver;
   }

   int wintv_pci_fm::create_i2c_devices(void)
   {
      m_i2c_devs[0].conf.in_addr = 0x61;
      m_i2c_devs[0].source_mask = CX88_SOURCE_DEFAULT;
      m_i2c_devs[1].conf.in_addr = 0x43;
      m_i2c_devs[1].source_mask = CX88_SOURCE_DEFAULT;
      return 0;
   }

   int wintv_pci_fm::select_source(uint32_t source_mask)
   {
      switch (source_mask)
      {
         case CX88_SOURCE_DEFAULT:
            m_input_mux = CX88_VIDEO_INPUT_MUX1;
            m_input_type = CX88_VIDEO_INPUT_COMPOSITE;
            m_gpio = 0x0;
            break;
         case PCIFM_SOURCE_SVIDEO:
            m_input_mux = CX88_VIDEO_INPUT_MUX3;
            m_input_type = CX88_VIDEO_INPUT_SVIDEO;
            m_gpio = 0x2;
            break;
         case (PCIFM_SOURCE_COMPOSITE | PCIFM_SOURCE_LINEIN):
            m_input_mux = CX88_VIDEO_INPUT_MUX2;
            m_input_type = CX88_VIDEO_INPUT_COMPOSITE;
            m_gpio = 0x2;
            break;
         case PCIFM_SOURCE_COMPOSITE:
            m_input_mux = CX88_VIDEO_INPUT_MUX2;
            m_input_type = CX88_VIDEO_INPUT_COMPOSITE;
            m_gpio = 0x0;
            break;
         case PCIFM_SOURCE_LINEIN:
            m_gpio = 0x2;
            break;
         default:
            return EINVAL;
      }
      return 0;
   }

   int wintv_pci_fm::set_capture_options(cx88_video_capture &capture)
   {
      capture.in_input_source = m_input_mux;
      capture.in_input_type = m_input_type;
      return 0;
   }

   int wintv_pci_fm::set_channel_options(avb_channel &channel)
   {
      switch (channel.audio_format)
      {
         case AVB_AUDIO_FMT_FM_MONO:
         case AVB_AUDIO_FMT_FM_MONO_NON_USA:
         case AVB_AUDIO_FMT_FM_MONO_USA:
         case AVB_AUDIO_FMT_FM_STEREO:
         case AVB_AUDIO_FMT_FM_STEREO_NON_USA:
         case AVB_AUDIO_FMT_FM_STEREO_USA:
            m_gpio = 0x1;
         default:
            break;
      }
      int error = 0;
      cx88_common_gpio gpio;
      gpio.in_write_mask = 0xffffff;
      gpio.in_read_mask = 0x0;
      gpio.inout_gpio = m_gpio;
      if ((error = ioctl(m_fd, CX88_COMMON_GPIO, &gpio)) != 0)
      {
         CX88ERR << "Error configuring GPIO lines: " << strerror(errno) << std::endl;
      }
      return error;
   }
}
